

#include <stdio.h>
#include <unistd.h>
#include <strings.h>

#include "findIt.h"


extern char *getenv();
extern char *strtok();
extern char *strdup();


/*********  globals  ************/


static char *Locales[32];		/* master list of places to look */


char *StdEnvItems[] = {		/* environment items with paths in them */
   "PWD",			/* this one is typical for hydra */
   "HOME",
   "FIBUSR",
   "FIBFIELDS",
   "FIBDATA",
   "FIBBIN",
   "PATH",
    NULL
};


/********* a test program ***********/

#ifdef TEST_ONLY

/* a test program for the  file */

/*ARGSUSED*/

main (argc, argv, envp)
    int argc;
    char **argv;
    char **envp;

{
   char *goodPath;

   goodPath = findFile( argv[1], StdEnvItems );

   if( goodPath != NULL )
      puts( goodPath );
   else
      puts( "Couldn't find it" );

   return ( 0 );
}

#endif

/********** the library **********/

/*
		findFile

   Take the specified file and find the path to it
   by searching the paths given by the specified
   environment list.  If the file is found,
   return a pointer to the complete path.  Else,
   return a null pointer.
*/

char *findFile( char *name, char **envList )

{
   static char testPath[255];
   int i = 0;
   int foundIt = 0;

   /* use the most recent environment */

   loadLocales( Locales, envList );

   /* start trying the possibilities */

   while( Locales[i] != NULL ){
      
      (void)strcpy( testPath, Locales[i] );
      (void)strcat( testPath, "/" );
      (void)strcat( testPath, name );

      if( access( testPath, F_OK ) == 0 ){
	 foundIt = TRUE;
	 break;
      }
      else
	 i++;
   }
   
   /* free up the strings we allocated */

   i = 0;

   while( Locales[i] != NULL )
      free( Locales[i++] );

   if( foundIt )
      return( testPath );
   else
      return ( (char *)NULL );
}

/*
		loadLocales

   Take a list of environment variables (assumed to contain
   paths, singularly or in combination) and build up an
   array or pointers to the names of places to look for a
   file.  This array is ordered the same as the environment
   list fed to it.  Both lists are terminated with a null pointer.
*/


void loadLocales( char **locales, char **envList )

{
   char *envItem;
   char *newCopy;

   int i = 0;
   int j = 0;

   /* loop through the list of environment items */

   while( envList[ i ] != NULL ){
      
      envItem = getenv( envList[ i++ ] );

      if( envItem != NULL ){

         /* the environment entry was there */

	 newCopy = strdup( envItem );

	 /* Now copy the first path in the item into the list */

	 if( ( locales[ j ] = strtok( newCopy, DELIMITER ) ) != NULL ){

	    j++;

	    /* see if there are any more paths on the linel... */

	    while( ( locales[ j ] = strtok( (char*)NULL, DELIMITER ) ) != NULL )
	       j++;
	 }	
      }
   }
   locales[ j ] = NULL;
}
